//******************************************************************************************
//* Calabi-Yau Manifold, by Paul Nylander, bugman123.com, 8/25/08
//* animation settings: +a0.3 +am2 +r1 -p +KI0.0 +KF1.0 +KFI1 +KFF180 +KC
//******************************************************************************************

#declare n=5;
#declare Alpha=(0.25+clock)*pi;

camera{orthographic location <2,2,0> look_at <0,0,0> right x*image_width/image_height up y sky y angle 75}
background{rgb <0,0,0>}
light_source{<2,2,0>,2}

//Complex Functions
#macro square(X) X*X #end
#macro Complex(X,Y) <X,Y> #end
#declare I=Complex(0,1);
#macro Re(Z) Z.x #end
#macro Im(Z) Z.y #end
#macro Abs(Z) vlength(Z) #end // sqrt(square(Re(Z))+square(Im(Z)))
#macro Arg(Z) #if(Abs(Z)=0) #local theta=0; #else #local theta=atan2(Im(Z),Re(Z)); #end theta #end
#macro Sqr(Z) Complex(square(Re(Z))-square(Im(Z)),2*Re(Z)*Im(Z)) #end
#macro Pow(Z,n) #local r=Abs(Z); #if(r=0) #local z2=Complex(0,0); #else #local theta=n*Arg(Z); #local z2=pow(r,n)*Complex(cos(theta),sin(theta)); #end z2 #end
#macro Mult(z1,z2) Complex(Re(z1)*Re(z2)-Im(z1)*Im(z2),Im(z1)*Re(z2)+Re(z1)*Im(z2)) #end
#macro Exp(Z) exp(Re(Z))*Complex(cos(Im(Z)),sin(Im(Z))) #end
#macro Sinh(Z) Complex(cos(Im(Z))*sinh(Re(Z)),sin(Im(Z))*cosh(Re(Z))) #end
#macro Cosh(Z) Complex(cos(Im(Z))*cosh(Re(Z)),sin(Im(Z))*sinh(Re(Z))) #end

//Parametric Plotting, function f must be defined before this function can be called
#macro ParametricPlot3D(u1,u2,du, v1,v2,dv) #local imax=int((u2-u1)/du+0.5); #local jmax=int((v2-v1)/dv+0.5);
 #local Mesh=array[imax+1][jmax+1]; #local i=0;
 #while(i<=imax) #local U=u1+i*du; #local j=0; #while(j<=jmax) #local Mesh[i][j]=f(U,v1+j*dv); #local j=j+1; #end #local i=i+1; #end
 DrawMesh(Mesh)
#end
#macro quad(p1,n1, p2,n2, p3,n3, p4,n4)
 #if(vlength(p3-p1)<vlength(p4-p2))
  smooth_triangle{p1,n1, p2,n2, p3,n3} smooth_triangle{p1,n1, p3,n3, p4,n4}
 #else
  smooth_triangle{p1,n1, p2,n2, p4,n4} smooth_triangle{p2,n2, p3,n3, p4,n4}
 #end
#end
#macro DrawMesh(Mesh)
 #local imax=dimension_size(Mesh,1); #local jmax=dimension_size(Mesh,2); #local normals=array[imax][jmax];
 #local i=0; // initialize normals
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=<0,0,0>; #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while(i<imax-1) #local j=0; // adapted from Tim Wenclawiak's Make_Normals function
  #while(j<jmax-1)
   #local p0=(Mesh[i][j]+Mesh[i][j+1]+Mesh[i+1][j]+Mesh[i+1][j+1])/4;
   #local N=vcross(Mesh[i+1][j]-p0,Mesh[i+1][j+1]-p0);
   #local E=vcross(Mesh[i+1][j+1]-p0,Mesh[i][j+1]-p0);
   #local S=vcross(Mesh[i][j+1]-p0,Mesh[i][j]-p0);
   #local W=vcross(Mesh[i][j]-p0,Mesh[i+1][j]-p0);
   #local normals[i][j]=normals[i][j]+S+W;
   #local normals[i+1][j]=normals[i+1][j]+N+W;
   #local normals[i][j+1]=normals[i][j+1]+S+E;
   #local normals[i+1][j+1]=normals[i+1][j+1]+N+E;
   #local j=j+1;
  #end
  #local i=i+1;
 #end
 #local i=0;
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=vnormalize(normals[i][j]); #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while (i<imax-1) #local j=0;
  #while (j<jmax-1)
   quad(Mesh[i][j],normals[i][j], Mesh[i+1][j],normals[i+1][j], Mesh[i+1][j+1],normals[i+1][j+1], Mesh[i][j+1],normals[i][j+1])
   #local j=j+1;
  #end
  #local i=i+1;
 #end
#end

//create the Calabi-Yau Manifold
#macro f(X,Y) #local Z=Complex(X,Y);
 #local z1=Mult(Exp(2*pi*I*k1/n),Pow(Cosh(Z),2/n));
 #local z2=Mult(Exp(2*pi*I*k2/n),Pow(Sinh(Z),2/n));
 #local p=<Re(z1),Re(z2),cos(Alpha)*Im(z1)+sin(Alpha)*Im(z2)>;
 <p.x,p.y,p.z>
#end
#declare k1=0;
#while(k1<n) #declare k2=0;
 #while(k2<n)
  union{ParametricPlot3D(-1,1,0.2, 0,pi/2,pi/45)
   pigment{function{atan2(y-x,sqrt(2)*z)/(2*pi)} color_map{[0 rgbt <1,0,0,0.5>][1/6 rgbt <1,1,0,0.5>][1/3 rgbt <0,1,0,0.5>][1/2 rgbt <0,1,1,0.5>][2/3 rgbt <0,0,1,0.5>][5/6 rgbt <1,0,1,0.5>][1 rgbt <1,0,0,0.5>]}} 
   finish{reflection 0.4}
  }
  #declare k2=k2+1;
 #end
 #declare k1=k1+1;
#end
